@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -6,8 +6,12 @@
 
 import re
 import time
+import socket
 
 try:
+    from ifupdown2.lib.addon import Addon
+    from ifupdown2.lib.log import LogManager
+
     import ifupdown2.ifupdown.policymanager as policymanager
     import ifupdown2.ifupdown.ifupdownflags as ifupdownflags
 
@@ -15,9 +19,11 @@ try:
     from ifupdown2.ifupdown.utils import utils
 
     from ifupdown2.ifupdownaddons.dhclient import dhclient
-    from ifupdown2.ifupdownaddons.LinkUtils import LinkUtils
     from ifupdown2.ifupdownaddons.modulebase import moduleBase
-except ImportError:
+except (ImportError, ModuleNotFoundError):
+    from lib.addon import Addon
+    from lib.log import LogManager
+
     import ifupdown.policymanager as policymanager
     import ifupdown.ifupdownflags as ifupdownflags
 
@@ -25,17 +31,43 @@ except ImportError:
     from ifupdown.utils import utils
 
     from ifupdownaddons.dhclient import dhclient
-    from ifupdownaddons.LinkUtils import LinkUtils
     from ifupdownaddons.modulebase import moduleBase
 
 
-class dhcp(moduleBase):
+class dhcp(Addon, moduleBase):
     """ ifupdown2 addon module to configure dhcp on interface """
 
+    # by default we won't perform any dhcp retry
+    # this can be changed by setting the module global
+    # policy: dhclient_retry_on_failure
+    DHCLIENT_RETRY_ON_FAILURE = 0
+
     def __init__(self, *args, **kargs):
+        Addon.__init__(self)
         moduleBase.__init__(self, *args, **kargs)
         self.dhclientcmd = dhclient(**kargs)
-        self.ipcmd = None
+
+        vrf_id = self._get_vrf_context()
+        if vrf_id and vrf_id == 'mgmt':
+            self.mgmt_vrf_context = True
+        else:
+            self.mgmt_vrf_context = False
+        self.logger.info('mgmt vrf_context = %s' %self.mgmt_vrf_context)
+
+        try:
+            self.dhclient_retry_on_failure = int(
+                policymanager.policymanager_api.get_module_globals(
+                    module_name=self.__class__.__name__,
+                    attr="dhclient_retry_on_failure"
+                )
+            )
+        except Exception:
+            self.dhclient_retry_on_failure = self.DHCLIENT_RETRY_ON_FAILURE
+
+        if self.dhclient_retry_on_failure < 0:
+            self.dhclient_retry_on_failure = 0
+
+        self.logger.info("dhclient: dhclient_retry_on_failure set to %s" % self.dhclient_retry_on_failure)
 
     def syntax_check(self, ifaceobj, ifaceobj_getfunc):
         return self.is_dhcp_allowed_on(ifaceobj, syntax_check=True)
@@ -45,6 +77,59 @@ class dhcp(moduleBase):
             return utils.is_addr_ip_allowed_on(ifaceobj, syntax_check=True)
         return True
 
+    def get_current_ip_configured(self, ifname, family):
+        ips = set()
+        try:
+            a = utils.exec_commandl(["ip", "-o", "addr", "show", ifname]).split("\n")
+
+            for entry in a:
+                family_index = entry.find(family)
+
+                if family_index < 0:
+                    continue
+
+                tmp = entry[entry.find(family) + len(family) + 1:]
+                ip = tmp[:tmp.find(" ")]
+
+                if ip:
+                    ips.add(ip)
+        except Exception:
+            pass
+        return ips
+
+    def dhclient_start_and_check(self, ifname, family, handler, **handler_kwargs):
+        ip_config_before = self.get_current_ip_configured(ifname, family)
+        retry = self.dhclient_retry_on_failure
+
+        while retry >= 0:
+            handler(ifname, **handler_kwargs)
+            retry = self.dhclient_check(ifname, family, ip_config_before, retry, handler_kwargs.get("cmd_prefix"))
+
+    def dhclient_check(self, ifname, family, ip_config_before, retry, dhclient_cmd_prefix):
+        retry -= 1
+        diff = self.get_current_ip_configured(ifname, family).difference(ip_config_before)
+
+        if diff:
+            self.logger.info(
+                "%s: dhclient: new address%s detected: %s"
+                % (ifname, "es" if len(diff) > 1 else "", ", ".join(diff))
+            )
+            return -1
+        else:
+            try:
+                if retry > 0:
+                    self.logger.error(
+                        "%s: dhclient: couldn't detect new ip address, retrying %s more times..."
+                        % (ifname, retry)
+                    )
+                else:
+                    self.logger.error("%s: dhclient: timeout failed to detect new ip addresses" % ifname)
+                    return -1
+            finally:
+                self.logger.info("%s: releasing expired dhcp lease..." % ifname)
+                self.dhclientcmd.release(ifname, dhclient_cmd_prefix)
+        return retry
+
     def _up(self, ifaceobj):
         # if dhclient is already running do not stop and start it
         dhclient4_running = self.dhclientcmd.is_running(ifaceobj.name)
@@ -56,6 +141,10 @@ class dhcp(moduleBase):
         self._down_stale_dhcp_config(ifaceobj, 'inet', dhclient4_running)
         self._down_stale_dhcp_config(ifaceobj, 'inet6', dhclient6_running)
 
+        if ifaceobj.link_privflags & ifaceLinkPrivFlags.KEEP_LINK_DOWN:
+            self.logger.info("%s: skipping dhcp configuration: link-down yes" % ifaceobj.name)
+            return
+
         try:
             dhclient_cmd_prefix = None
             dhcp_wait = policymanager.policymanager_api.get_attr_default(
@@ -65,14 +154,18 @@ class dhcp(moduleBase):
                 ifname=ifaceobj.name, attr='dhcp6-ll-wait')
             try:
                 timeout = int(dhcp6_ll_wait)+1
-            except:
+            except Exception:
                 timeout = 10
                 pass
-
+            dhcp6_duid = policymanager.policymanager_api.get_iface_default(module_name=self.__class__.__name__, \
+                ifname=ifaceobj.name, attr='dhcp6-duid')
             vrf = ifaceobj.get_attr_value_first('vrf')
             if (vrf and self.vrf_exec_cmd_prefix and
-                self.ipcmd.link_exists(vrf)):
+                self.cache.link_exists(vrf)):
                 dhclient_cmd_prefix = '%s %s' %(self.vrf_exec_cmd_prefix, vrf)
+            elif self.mgmt_vrf_context:
+                dhclient_cmd_prefix = '%s %s' %(self.vrf_exec_cmd_prefix, 'default')
+                self.logger.info('detected mgmt vrf context starting dhclient in default vrf context')
 
             if 'inet' in ifaceobj.addr_family:
                 if dhclient4_running:
@@ -83,10 +176,17 @@ class dhcp(moduleBase):
                     try:
                         if not ifupdownflags.flags.PERFMODE:
                             self.dhclientcmd.stop(ifaceobj.name)
-                    except:
+                    except Exception:
                         pass
-                    self.dhclientcmd.start(ifaceobj.name, wait=wait,
-                                           cmd_prefix=dhclient_cmd_prefix)
+
+                    self.dhclient_start_and_check(
+                        ifaceobj.name,
+                        "inet",
+                        self.dhclientcmd.start,
+                        wait=wait,
+                        cmd_prefix=dhclient_cmd_prefix
+                    )
+
             if 'inet6' in ifaceobj.addr_family:
                 if dhclient6_running:
                     self.logger.info('dhclient6 already running on %s. '
@@ -103,14 +203,13 @@ class dhcp(moduleBase):
                         self.sysctl_set('net.ipv6.conf.%s' %ifaceobj.name +
                                 '.autoconf', autoconf)
                         try:
-                            self.dhclientcmd.stop6(ifaceobj.name)
-                        except:
+                            self.dhclientcmd.stop6(ifaceobj.name, duid=dhcp6_duid)
+                        except Exception:
                             pass
                     #add delay before starting IPv6 dhclient to
                     #make sure the configured interface/link is up.
                     if timeout > 1:
-                       time.sleep(1)
-
+                        time.sleep(1)
                     while timeout:
                         addr_output = utils.exec_command('%s -6 addr show %s'
                                                          %(utils.ip_cmd, ifaceobj.name))
@@ -118,14 +217,14 @@ class dhcp(moduleBase):
                         if r:
                             self.dhclientcmd.start6(ifaceobj.name,
                                                     wait=wait,
-                                                    cmd_prefix=dhclient_cmd_prefix)
+                                                    cmd_prefix=dhclient_cmd_prefix, duid=dhcp6_duid)
                             return
                         timeout -= 1
                         if timeout:
                             time.sleep(1)
-
-        except Exception, e:
-            self.log_error(str(e), ifaceobj)
+        except Exception as e:
+            self.logger.error("%s: %s" % (ifaceobj.name, str(e)))
+            ifaceobj.set_status(ifaceStatus.ERROR)
 
     def _down_stale_dhcp_config(self, ifaceobj, family, dhclientX_running):
         addr_family = ifaceobj.addr_family
@@ -133,7 +232,7 @@ class dhcp(moduleBase):
             if not family in ifaceobj.addr_family and dhclientX_running:
                 ifaceobj.addr_family = [family]
                 self._dhcp_down(ifaceobj)
-        except:
+        except Exception:
             pass
         finally:
             ifaceobj.addr_family = addr_family
@@ -142,16 +241,20 @@ class dhcp(moduleBase):
         dhclient_cmd_prefix = None
         vrf = ifaceobj.get_attr_value_first('vrf')
         if (vrf and self.vrf_exec_cmd_prefix and
-            self.ipcmd.link_exists(vrf)):
+            self.cache.link_exists(vrf)):
             dhclient_cmd_prefix = '%s %s' %(self.vrf_exec_cmd_prefix, vrf)
+        dhcp6_duid = policymanager.policymanager_api.get_iface_default(module_name=self.__class__.__name__, \
+                                                                       ifname=ifaceobj.name, attr='dhcp6-duid')
         if 'inet6' in ifaceobj.addr_family:
-            self.dhclientcmd.release6(ifaceobj.name, dhclient_cmd_prefix)
+            self.dhclientcmd.release6(ifaceobj.name, dhclient_cmd_prefix, duid=dhcp6_duid)
+            self.cache.force_address_flush_family(ifaceobj.name, socket.AF_INET6)
         if 'inet' in ifaceobj.addr_family:
             self.dhclientcmd.release(ifaceobj.name, dhclient_cmd_prefix)
+            self.cache.force_address_flush_family(ifaceobj.name, socket.AF_INET)
 
     def _down(self, ifaceobj):
         self._dhcp_down(ifaceobj)
-        self.ipcmd.link_down(ifaceobj.name)
+        self.netlink.link_down(ifaceobj.name)
 
     def _query_check(self, ifaceobj, ifaceobjcurr):
         status = ifaceStatus.SUCCESS
@@ -177,7 +280,7 @@ class dhcp(moduleBase):
         ifaceobjcurr.status = status
 
     def _query_running(self, ifaceobjrunning):
-        if not self.ipcmd.link_exists(ifaceobjrunning.name):
+        if not self.cache.link_exists(ifaceobjrunning.name):
             return
         if self.dhclientcmd.is_running(ifaceobjrunning.name):
             ifaceobjrunning.addr_family.append('inet')
@@ -194,11 +297,7 @@ class dhcp(moduleBase):
 
     def get_ops(self):
         """ returns list of ops supported by this module """
-        return self._run_ops.keys()
-
-    def _init_command_handlers(self):
-        if not self.ipcmd:
-            self.ipcmd = LinkUtils()
+        return list(self._run_ops.keys())
 
     def run(self, ifaceobj, operation, query_ifaceobj=None, **extra_args):
         """ run dhcp configuration on the interface object passed as argument
@@ -225,12 +324,43 @@ class dhcp(moduleBase):
                    (ifaceobj.addr_method != 'dhcp' and
                        ifaceobj.addr_method != 'dhcp6')):
                 return
-        except:
+        except Exception:
             return
         if not self.is_dhcp_allowed_on(ifaceobj, syntax_check=False):
             return
-        self._init_command_handlers()
-        if operation == 'query-checkcurr':
-            op_handler(self, ifaceobj, query_ifaceobj)
-        else:
-            op_handler(self, ifaceobj)
+
+        log_manager = LogManager.get_instance()
+
+        syslog_log_level = logging.INFO
+        disable_syslog_on_exit = None
+
+        if operation in ["up", "down"]:
+            # if syslog is already enabled we shouldn't disable it
+            if log_manager.is_syslog_enabled():
+                # save current syslog level
+                syslog_log_level = log_manager.get_syslog_log_level()
+                # prevent syslog from being disabled on exit
+                disable_syslog_on_exit = False
+            else:
+                # enabling syslog
+                log_manager.enable_syslog()
+                # syslog will be disabled once we are done
+                disable_syslog_on_exit = True
+
+            # update the current syslog handler log level if higher than INFO
+            if syslog_log_level >= logging.INFO:
+                log_manager.set_level_syslog(logging.INFO)
+
+            self.logger.info("%s: enabling syslog for dhcp configuration" % ifaceobj.name)
+
+        try:
+            if operation == 'query-checkcurr':
+                op_handler(self, ifaceobj, query_ifaceobj)
+            else:
+                op_handler(self, ifaceobj)
+        finally:
+            # disable syslog handler or re-set the proper log-level
+            if disable_syslog_on_exit is True:
+                log_manager.get_instance().disable_syslog()
+            elif disable_syslog_on_exit is False:
+                log_manager.set_level_syslog(syslog_log_level)
