@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -7,69 +7,170 @@
 import os
 
 try:
+    from ifupdown2.lib.addon import Addon
+
+    import ifupdown2.ifupdown.ifupdownflags as ifupdownflags
     import ifupdown2.ifupdown.ifupdownflags as ifupdownflags
     import ifupdown2.ifupdown.policymanager as policymanager
+    import ifupdown2.ifupdown.statemanager as statemanager
 
     from ifupdown2.ifupdown.iface import *
     from ifupdown2.ifupdown.utils import utils
     from ifupdown2.ifupdown.exceptions import moduleNotSupported
 
     from ifupdown2.ifupdownaddons.utilsbase import *
-    from ifupdown2.ifupdownaddons.LinkUtils import LinkUtils
     from ifupdown2.ifupdownaddons.modulebase import moduleBase
-except ImportError:
+except (ImportError, ModuleNotFoundError):
+    from lib.addon import Addon
+
     import ifupdown.ifupdownflags as ifupdownflags
     import ifupdown.policymanager as policymanager
+    import ifupdown.statemanager as statemanager
 
     from ifupdown.iface import *
     from ifupdown.utils import utils
     from ifupdown.exceptions import moduleNotSupported
 
     from ifupdownaddons.utilsbase import *
-    from ifupdownaddons.LinkUtils import LinkUtils
     from ifupdownaddons.modulebase import moduleBase
 
 
-class ethtool(moduleBase,utilsBase):
+class ethtool(Addon, moduleBase):
     """  ifupdown2 addon module to configure ethtool attributes """
 
-    _modinfo = {'mhelp' : 'ethtool configuration module for interfaces',
-                'attrs': {
-                      'link-speed' :
-                            {'help' : 'set link speed',
-                             'validvals' : ['10',
-                                            '100',
-                                            '1000',
-                                            '10000',
-                                            '25000',
-                                            '40000',
-                                            '50000',
-                                            '100000'],
-                             'example' : ['link-speed 1000'],
-                             'default' : 'varies by platform and port'},
-                      'link-duplex' :
-                            {'help': 'set link duplex',
-                             'example' : ['link-duplex full'],
-                             'validvals' : ['half', 'full'],
-                             'default' : 'full'},
-                      'link-autoneg' :
-                            {'help': 'set autonegotiation',
-                             'example' : ['link-autoneg on'],
-                             'validvals' : ['yes', 'no', 'on', 'off'],
-                             'default' : 'varies by platform and port'},
-                      'link-fec' :
-                            {'help': 'set forward error correction mode',
-                             'example' : ['link-fec rs'],
-                             'validvals' : ['rs', 'baser', 'auto', 'off'],
-                             'default' : 'varies by platform and port'}}}
+    _modinfo = {
+        "mhelp": "ethtool configuration module for interfaces",
+        "attrs": {
+            "link-speed": {
+                "help": "set link speed",
+                "validvals": [
+                    "10",
+                    "100",
+                    "1000",
+                    "10000",
+                    "25000",
+                    "40000",
+                    "50000",
+                    "100000",
+                    "200000",
+                    "400000"
+                ],
+                "example": ["link-speed 1000"],
+                "default": "varies by platform and port"
+            },
+            "link-duplex": {
+                "help": "set link duplex",
+                "example": ["link-duplex full"],
+                "validvals": ["half", "full"],
+                "default": "full"
+            },
+            "link-autoneg": {
+                "help": "set autonegotiation",
+                "example": ["link-autoneg on"],
+                "validvals": ["yes", "no", "on", "off"],
+                "default": "varies by platform and port"
+            },
+            "link-fec": {
+                "help": "set forward error correction mode",
+                "example": ["link-fec rs"],
+                "validvals": ["rs", "baser", "auto", "off"],
+                "default": "varies by platform and port"
+            },
+            'gro-offload': {
+                'help': 'Generic Receive Offload',
+                'example': ['gro-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'lro-offload': {
+                'help': 'Large Receive Offload',
+                'example': ['lro-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'gso-offload': {
+                'help': 'Generic Segmentation Offload',
+                'example': ['tso-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'tso-offload': {
+                'help': 'TCP Segmentation Offload',
+                'example': ['tso-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'ufo-offload': {
+                'help': 'UDP Fragmentation Offload',
+                'example': ['ufo-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'tx-offload': {
+                'help': 'TX Checksum Offload',
+                'example': ['tx-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+            'rx-offload': {
+                'help': 'RX Checksum Offload',
+                'example': ['rx-offload on'],
+                'validvals': ['on', 'off'],
+                'default': 'varies by interface'
+            },
+        }
+    }
 
     def __init__(self, *args, **kargs):
+        Addon.__init__(self)
         moduleBase.__init__(self, *args, **kargs)
         if not os.path.exists(utils.ethtool_cmd):
             raise moduleNotSupported('module init failed: %s: not found' % utils.ethtool_cmd)
-        self.ipcmd = None
         # keep a list of iface objects who have modified link attributes
         self.ifaceobjs_modified_configs = []
+        # Cache for features
+        self.feature_cache = None
+
+        self.ethtool_ignore_errors = policymanager.policymanager_api.get_module_globals(
+            module_name=self.__class__.__name__,
+            attr='ethtool_ignore_errors'
+        )
+
+    def do_offload_settings(self, ifaceobj, attr_name, eth_name):
+        default = 'default_' + eth_name
+        config_val = ifaceobj.get_attr_value_first(attr_name)
+        # Default
+        default_val = None
+        saved_ifaceobjs = statemanager.statemanager_api.get_ifaceobjs(ifaceobj.name)
+        if saved_ifaceobjs:
+            default_val = saved_ifaceobjs[0].get_attr_value_first(default)
+        if config_val or default_val:
+
+            # get running value
+            running_val = str(self.get_running_attr(eth_name, ifaceobj)).lower()
+            # Save default value
+            # Load state data
+            if not default_val:
+                ifaceobj.config[default] = [running_val]
+            elif config_val:
+                # resave for state
+                ifaceobj.config[default] = [default_val]
+
+            if not config_val:
+                config_val = default_val
+
+            if config_val and config_val != running_val:
+                try:
+                    cmd = ('%s -K %s %s %s' %
+                            (utils.ethtool_cmd, ifaceobj.name, eth_name, config_val))
+                    utils.exec_command(cmd)
+                except Exception as e:
+                    self.log_error('%s: %s' %(ifaceobj.name, str(e)), ifaceobj)
+
+        self.ethtool_ignore_errors = policymanager.policymanager_api.get_module_globals(
+            module_name=self.__class__.__name__,
+            attr='ethtool_ignore_errors'
+        )
 
     def do_fec_settings(self, ifaceobj):
         feccmd = ''
@@ -94,10 +195,6 @@ class ethtool(moduleBase,utilsBase):
         if default_val:
             default_val = default_val.lower()
 
-        if running_val in ["none", "notsupported"]:
-            # None and NotSupported ethtool FEC values mean "off"
-            running_val = "off"
-
         # check running values
         if config_val and config_val == running_val:
             return
@@ -119,8 +216,9 @@ class ethtool(moduleBase,utilsBase):
                 feccmd = ('%s --set-fec %s %s' %
                            (utils.ethtool_cmd, ifaceobj.name, feccmd))
                 utils.exec_command(feccmd)
-            except Exception, e:
-                self.log_error('%s: %s' %(ifaceobj.name, str(e)), ifaceobj)
+            except Exception as e:
+                if not self.ethtool_ignore_errors:
+                    self.log_error('%s: %s' %(ifaceobj.name, str(e)), ifaceobj)
         else:
             pass
 
@@ -213,19 +311,27 @@ class ethtool(moduleBase,utilsBase):
             try:
                 cmd = ('%s -s %s %s' % (utils.ethtool_cmd, ifaceobj.name, cmd))
                 utils.exec_command(cmd)
-            except Exception, e:
-                self.log_error('%s: %s' % (ifaceobj.name, str(e)), ifaceobj)
+            except Exception as e:
+                if not self.ethtool_ignore_errors:
+                    self.log_error('%s: %s' % (ifaceobj.name, str(e)), ifaceobj)
 
     def _pre_up(self, ifaceobj, operation='post_up'):
         """
         _pre_up and _pre_down will reset the layer 2 attributes to default policy
         settings.
         """
-        if not self.ipcmd.link_exists(ifaceobj.name):
+        if not self.cache.link_exists(ifaceobj.name):
             return
 
         self.do_speed_settings(ifaceobj)
         self.do_fec_settings(ifaceobj)
+        self.do_offload_settings(ifaceobj, 'gro-offload', 'gro')
+        self.do_offload_settings(ifaceobj, 'lro-offload', 'lro')
+        self.do_offload_settings(ifaceobj, 'gso-offload', 'gso')
+        self.do_offload_settings(ifaceobj, 'tso-offload', 'tso')
+        self.do_offload_settings(ifaceobj, 'ufo-offload', 'ufo')
+        self.do_offload_settings(ifaceobj, 'tx-offload', 'tx')
+        self.do_offload_settings(ifaceobj, 'rx-offload', 'rx')
 
     def _pre_down(self, ifaceobj):
         pass #self._post_up(ifaceobj,operation="_pre_down")
@@ -306,15 +412,30 @@ class ethtool(moduleBase,utilsBase):
         """
         try:
             for attr in ethtool_output.splitlines():
-                if attr.startswith('FEC encodings'):
+                if attr.startswith('Configured FEC encodings:'):
                     fec_attrs = attr.split()
-                    return(fec_attrs[fec_attrs.index(':')+1])
+                    return(fec_attrs[fec_attrs.index('encodings:')+1])
         except Exception as e:
             self.logger.debug('ethtool: problems in ethtool set-fec output'
                                ' %s: %s' %(ethtool_output.splitlines(), str(e)))
 
         return(None)
 
+    def get_offload_setting(self, ethtool_output, setting):
+
+        value = None
+
+        for line in ethtool_output.splitlines():
+            if setting in line:
+                if 'on' in line:
+                    value = 'on'
+                elif 'off' in line:
+                    value = 'off'
+
+                break
+
+        return value
+
     def get_running_attr(self,attr='',ifaceobj=None):
         if not ifaceobj or not attr:
             return
@@ -327,14 +448,50 @@ class ethtool(moduleBase,utilsBase):
                 output = utils.exec_command('%s --show-fec %s'%
                                             (utils.ethtool_cmd, ifaceobj.name))
                 running_attr = self.get_fec_encoding(ethtool_output=output)
+            elif attr == 'gro':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='generic-receive-offload')
+            elif attr == 'lro':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='large-receive-offload')
+            elif attr == 'gso':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='generic-segmentation-offload')
+            elif attr == 'tso':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='tcp-segmentation-offload')
+            elif attr == 'ufo':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='udp-fragmentation-offload')
+            elif attr == 'rx':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='rx-checksumming')
+            elif attr == 'tx':
+                if not self.feature_cache:
+                    self.feature_cache = utils.exec_command('%s --show-features %s'%
+                                            (utils.ethtool_cmd, ifaceobj.name))
+                running_attr = self.get_offload_setting(ethtool_output=self.feature_cache, setting='tx-checksumming')
             else:
-                running_attr = self.read_file_oneline('/sys/class/net/%s/%s' % \
+                running_attr = self.io.read_file_oneline('/sys/class/net/%s/%s' % \
                                                       (ifaceobj.name, attr))
         except Exception as e:
-            # for nonexistent interfaces, we get an error (rc = 256 or 19200)
-            self.logger.debug('ethtool: problems calling ethtool or reading'
-                              ' /sys/class on iface %s for attr %s: %s' %
-                              (ifaceobj.name, attr, str(e)))
+            if not self.ethtool_ignore_errors:
+                # for nonexistent interfaces, we get an error (rc = 256 or 19200)
+                self.logger.debug('ethtool: problems calling ethtool or reading'
+                                  ' /sys/class on iface %s for attr %s: %s' %
+                                  (ifaceobj.name, attr, str(e)))
         return running_attr
 
 
@@ -347,7 +504,7 @@ class ethtool(moduleBase,utilsBase):
         """
         # do not bother showing swp ifaces that are not up for the speed
         # duplex and autoneg are not reliable.
-        if not self.ipcmd.is_link_up(ifaceobj.name):
+        if not self.cache.link_is_up(ifaceobj.name):
             return
         for attr in ['speed', 'duplex', 'autoneg']:
             default_val = policymanager.policymanager_api.get_iface_default(
@@ -394,11 +551,7 @@ class ethtool(moduleBase,utilsBase):
 
     def get_ops(self):
         """ returns list of ops supported by this module """
-        return self._run_ops.keys()
-
-    def _init_command_handlers(self):
-        if not self.ipcmd:
-            self.ipcmd = LinkUtils()
+        return list(self._run_ops.keys())
 
     def run(self, ifaceobj, operation, query_ifaceobj=None, **extra_args):
         """ run ethtool configuration on the interface object passed as
@@ -423,7 +576,6 @@ class ethtool(moduleBase,utilsBase):
         op_handler = self._run_ops.get(operation)
         if not op_handler:
             return
-        self._init_command_handlers()
         if operation == 'query-checkcurr':
             op_handler(self, ifaceobj, query_ifaceobj)
         else:
