@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -14,9 +14,12 @@ file. But can be extended to include any other network interface format
 """
 
 import json
+import logging
 
 from collections import OrderedDict
 
+log = logging.getLogger()
+
 
 class ifaceStatusUserStrs():
     """ This class declares strings user can see during an ifquery --check
@@ -47,16 +50,16 @@ class ifaceLinkKind():
         bond have an ifaceobj.role attribute of SLAVE and the bridge or
         bond itself has ifaceobj.role of MASTER.
     """
-    UNKNOWN         = 0x0000000
-    BRIDGE          = 0x0000001
-    BOND            = 0x0000010
-    VLAN            = 0x0000100
-    VXLAN           = 0x0001000
-    VRF             = 0x0010000
-    BATMAN_ADV      = 0x0100000
+    UNKNOWN = 0x000000
+    BRIDGE =  0x000001
+    BOND =    0x000010
+    VLAN =    0x000100
+    VXLAN =   0x001000
+    VRF =     0x010000
+    BATMAN_ADV = 0x0100000
     # to indicate logical interface created by an external entity.
     # the 'kind' of which ifupdown2 does not really understand
-    OTHER           = 0x1000000
+    OTHER =     0x100000
 
     @classmethod
     def to_str(cls, kind):
@@ -70,10 +73,13 @@ class ifaceLinkKind():
             return "vxlan"
         elif kind == cls.VRF:
             return "vrf"
+        else:
+            return "OTHER"
 
 class ifaceLinkPrivFlags():
     """ This corresponds to kernel netdev->priv_flags
         and can be BRIDGE_PORT, BOND_SLAVE etc """
+
     UNKNOWN =           0x00000
     BRIDGE_PORT =       0x00001
     BOND_SLAVE =        0x00010
@@ -83,36 +89,47 @@ class ifaceLinkPrivFlags():
     ADDRESS_VIRTUAL_SLAVE = 0x100000
     LOOPBACK = 0x1000000
     KEEP_LINK_DOWN = 0x10000000
+    MGMT_INTF = 0x100000000
+    SINGLE_VXLAN = 0x1000000000
+    ES_BOND = 0x10000000000
+    OPENVSWITCH = 0x10000000000
 
     @classmethod
     def get_str(cls, flag):
-        if flag == cls.UNKNOWN:
-            return 'unknown'
-        elif flag == cls.BRIDGE_PORT:
-            return 'bridge port'
-        elif flag == cls.BOND_SLAVE:
-            return 'bond slave'
-        elif flag == cls.VRF_SLAVE:
-            return 'vrf slave'
-        elif flag == cls.BRIDGE_VLAN_AWARE:
-            return 'vlan aware bridge'
-        elif flag == cls.BRIDGE_VXLAN:
-            return 'vxlan bridge'
+        string_list = []
+
+        if flag & cls.BRIDGE_PORT:
+            string_list.append("bridge port")
+
+        if flag & cls.BOND_SLAVE:
+            string_list.append("bond slave")
+
+        if flag & cls.VRF_SLAVE:
+            string_list.append("vrf slave")
+
+        if flag & cls.BRIDGE_VLAN_AWARE:
+            string_list.append("vlan aware bridge")
+
+        if flag & cls.BRIDGE_VXLAN:
+            string_list.append("vxlan bridge")
+
+        if flag & cls.ADDRESS_VIRTUAL_SLAVE:
+            string_list.append("address virtual slave")
+
+        if flag & cls.LOOPBACK:
+            string_list.append("loopback")
+
+        if flag & cls.KEEP_LINK_DOWN:
+            string_list.append("keep ling down")
+
+        if flag & cls.ES_BOND:
+            string_list.append("es bond")
+
+        if flag & cls.OPENVSWITCH:
+            string_list.append("openvswitch interface")
+
+        return ", ".join(string_list)
 
-    @classmethod
-    def get_all_str(cls, flags):
-        str = ''
-        if flags & cls.BRIDGE_PORT:
-            str += 'bridgeport '
-        if flags & cls.BOND_SLAVE:
-            str += 'bondslave '
-        if flags & cls.VRF_SLAVE:
-            str += 'vrfslave '
-        if flags & cls.BRIDGE_VLAN_AWARE:
-            str += 'vlanawarebridge '
-        if flags & cls.BRIDGE_VXLAN:
-            str += 'vxlanbridge '
-        return str
 
 class ifaceLinkType():
     LINK_UNKNOWN = 0x0
@@ -259,7 +276,7 @@ class ifaceJsonEncoder(json.JSONEncoder):
         retifacedict = OrderedDict([])
         if o.config:
             retconfig = dict((k, (v[0] if len(v) == 1 else v))
-                             for k,v in o.config.items())
+                             for k,v in list(o.config.items()))
         retifacedict['name'] = o.name
         if o.addr_method:
             if 'inet' in o.addr_family and 'dhcp' in o.addr_method:
@@ -282,7 +299,7 @@ class ifaceJsonEncoderWithStatus(json.JSONEncoder):
         retconfig_status = {}
         retifacedict = OrderedDict([])
         if o.config:
-            for k,v in o.config.items():
+            for k,v in list(o.config.items()):
                 idx = 0
                 vitem_status = []
                 for vitem in v:
@@ -324,8 +341,8 @@ class ifaceJsonDecoder():
     def json_to_ifaceobj(cls, ifaceattrdict):
         ifaceattrdict['config'] = OrderedDict([(k, (v if isinstance(v, list)
                                                 else [v.strip()]))
-                                for k,v in ifaceattrdict.get('config',
-                                            OrderedDict()).items()])
+                                for k,v in list(ifaceattrdict.get('config',
+                                            OrderedDict()).items())])
         return iface(attrsdict=ifaceattrdict)
 
 class iface():
@@ -525,7 +542,7 @@ class iface():
         if attr_value_list:
             try:
                 return attr_value_list[attr_index]
-            except:
+            except Exception:
                 return None
         return None
 
@@ -542,7 +559,7 @@ class iface():
         env = {}
         config = self.config
         env['IFACE'] = self.name
-        for attr, attr_value in config.items():
+        for attr, attr_value in list(config.items()):
             attr_env_name = 'IF_%s' %attr.upper().replace("-", "_")
             env[attr_env_name] = attr_value[0]
         self.env = env
@@ -559,7 +576,7 @@ class iface():
         """ add attribute name and value to the interface config """
         try:
             del self.config[attr_name]
-        except:
+        except Exception:
             pass
 
     def update_config_dict(self, attrdict):
@@ -610,13 +627,13 @@ class iface():
             return False
         if any(True for k in self.config if k not in dstiface.config):
             return False
-        if any(True for k,v in self.config.items()
+        if any(True for k,v in list(self.config.items())
                     if v != dstiface.config.get(k)): return False
         return True
 
     def squash(self, newifaceobj):
         """ This squashes the iface object """
-        for attrname, attrlist in newifaceobj.config.iteritems():
+        for attrname, attrlist in newifaceobj.config.items():
             # if allready present add it to the list
             # else add it to the end of the dictionary
             # We need to maintain order.
@@ -647,14 +664,13 @@ class iface():
         del odict['env']
         del odict['link_type']
         del odict['link_kind']
-        del odict['link_privflags']
+        #del odict['link_privflags']
         del odict['role']
         del odict['dependency_type']
         del odict['blacklisted']
         return odict
 
     def __setstate__(self, dict):
-        self.__dict__.update(dict)
         self._config_status = {}
         self.state = ifaceState.NEW
         self.status = ifaceStatus.UNKNOWN
@@ -674,14 +690,15 @@ class iface():
         self.link_privflags = ifaceLinkPrivFlags.UNKNOWN
         self.dependency_type = ifaceDependencyType.UNKNOWN
         self.blacklisted = False
+        self.__dict__.update(dict)
 
     def dump_raw(self, logger):
         indent = '  '
         if self.auto:
-            print 'auto %s' %self.name
-        print (self.raw_config[0])
+            print('auto %s' %self.name)
+        print((self.raw_config[0]))
         for i in range(1, len(self.raw_config)):
-            print(indent + self.raw_config[i])
+            print((indent + self.raw_config[i]))
 
     def dump(self, logger):
         indent = '\t'
@@ -706,6 +723,24 @@ class iface():
         else:
             logger.info(indent + 'upperdevs: None')
 
+        logger.info("%srole: %s" % (indent, {
+            ifaceRole.UNKNOWN: "UNKNOWN",
+            ifaceRole.SLAVE: "SLAVE",
+            ifaceRole.MASTER: "MASTER"}.get(self.role)))
+
+        logger.info("%stype: %s" % (indent, {
+            ifaceType.UNKNOWN: "UNKNOWN",
+            ifaceType.IFACE: "IFACE",
+            ifaceType.BRIDGE_VLAN: "BRIDGE_VLAN"}.get(self.type)))
+
+        logger.info("%slink_kind: %s" % (indent, ifaceLinkKind.to_str(self.link_kind)))
+
+        logger.info("%slink_privflags: %s" % (indent, ifaceLinkPrivFlags.get_str(self.link_privflags)))
+
+        if self.priv_flags:
+            if self.priv_flags.BUILTIN:
+                logger.info("%spriv_flags: BUILTIN" % indent)
+
         logger.info(indent + 'config: ')
         config = self.config
         if config:
@@ -744,15 +779,13 @@ class iface():
             else:
                 outbuf += ifaceline + '\n'
             if self.status == ifaceStatus.NOTFOUND:
-                outbuf = (outbuf.encode('utf8')
-                    if isinstance(outbuf, unicode) else outbuf)
-                print outbuf + '\n'
+                print(outbuf + '\n')
                 return
         else:
             outbuf += ifaceline + '\n'
         config = self.config
         if config and first:
-            for cname, cvaluelist in config.items():
+            for cname, cvaluelist in list(config.items()):
                 idx = 0
                 for cv in cvaluelist:
                     status_str = None
@@ -768,12 +801,10 @@ class iface():
                         outbuf += (indent + '{0:55} {1:>10}'.format(
                               '%s %s' %(cname, cv), status_str)) + '\n'
                     else:
-                        outbuf += indent + '%s %s\n' %(cname, cv)
+                        if cv:
+                            outbuf += indent + '%s %s\n' % (cname, cv)
                     idx += 1
-        if with_status:
-            outbuf = (outbuf.encode('utf8')
-                        if isinstance(outbuf, unicode) else outbuf)
-        print outbuf
+        print(outbuf)
 
     def dump_pretty(self, with_status=False, use_realname=False):
         if not self.addr_family:
