@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -17,7 +17,7 @@ try:
     from ifupdown2.ifupdown.iface import *
     from ifupdown2.ifupdown.utils import utils
     from ifupdown2.ifupdown.template import templateEngine
-except ImportError:
+except (ImportError, ModuleNotFoundError):
     from ifupdown.iface import *
     from ifupdown.utils import utils
     from ifupdown.template import templateEngine
@@ -30,11 +30,12 @@ class networkInterfaces():
 
     _addrfams = {'inet' : ['static', 'manual', 'loopback', 'dhcp', 'dhcp6', 'ppp', 'tunnel'],
                  'inet6' : ['static', 'manual', 'loopback', 'dhcp', 'dhcp6', 'ppp', 'tunnel']}
+    # tunnel is part of the address family for backward compatibility but is not required.
 
     def __init__(self, interfacesfile='/etc/network/interfaces',
                  interfacesfileiobuf=None, interfacesfileformat='native',
                  template_enable='0', template_engine=None,
-                 template_lookuppath=None):
+                 template_lookuppath=None, raw=False):
         """This member function initializes the networkinterfaces parser object.
 
         Kwargs:
@@ -54,7 +55,7 @@ class networkInterfaces():
         self.auto_ifaces = []
         self.callbacks = {}
         self.auto_all = False
-
+        self.raw = raw
         self.logger = logging.getLogger('ifupdown.' +
                     self.__class__.__name__)
         self.callbacks = {'iface_found' : None,
@@ -81,7 +82,7 @@ class networkInterfaces():
     def _currentfile(self):
         try:
             return self._filestack[-1]
-        except:
+        except Exception:
             return self.interfacesfile
 
     def _parse_error(self, filename, lineno, msg):
@@ -93,9 +94,9 @@ class networkInterfaces():
 
     def _parse_warn(self, filename, lineno, msg):
         if lineno == -1 or self._currentfile_has_template:
-            self.logger.warn('%s: %s' %(filename, msg))
+            self.logger.warning('%s: %s' %(filename, msg))
         else:
-            self.logger.warn('%s: line%d: %s' %(filename, lineno, msg))
+            self.logger.warning('%s: line%d: %s' %(filename, lineno, msg))
         self.warns += 1
 
     def _validate_addr_family(self, ifaceobj, lineno=-1):
@@ -127,8 +128,8 @@ class networkInterfaces():
         Warns on error
         """
 
-        if callback_name not in self.callbacks.keys():
-            print 'warning: invalid callback ' + callback_name
+        if callback_name not in list(self.callbacks.keys()):
+            print('warning: invalid callback ' + callback_name)
             return -1
 
         self.callbacks[callback_name] = callback_func
@@ -208,7 +209,7 @@ class networkInterfaces():
                         'iface %s: unsupported keyword (%s)'
                         %(ifacename, attrname))
                 return
-        except:
+        except Exception:
             pass
         attrvallist = iface_config.get(newattrname, [])
         if newattrname in ['scope', 'netmask', 'broadcast', 'preferred-lifetime']:
@@ -257,12 +258,14 @@ class networkInterfaces():
            self._parse_warn(self._currentfile, lineno,
                     '%s: unexpected characters in interface name' %ifacename)
 
-        ifaceobj.raw_config.append(iface_line)
+        if self.raw:
+            ifaceobj.raw_config.append(iface_line)
         iface_config = collections.OrderedDict()
         for line_idx in range(cur_idx + 1, len(lines)):
             l = lines[line_idx].strip(whitespaces)
             if self.ignore_line(l) == 1:
-                ifaceobj.raw_config.append(l)
+                if self.raw:
+                    ifaceobj.raw_config.append(l)
                 continue
             attrs = re.split(self._ws_split_regex, l, 1)
             if self._is_keyword(attrs[0]):
@@ -273,7 +276,8 @@ class networkInterfaces():
                 self._parse_error(self._currentfile, line_idx,
                         'iface %s: invalid syntax \'%s\'' %(ifacename, l))
                 continue
-            ifaceobj.raw_config.append(l)
+            if self.raw:
+                ifaceobj.raw_config.append(l)
             attrname = attrs[0]
             # preprocess vars (XXX: only preprocesses $IFACE for now)
             attrval = re.sub(r'\$IFACE', ifacename, attrs[1])
@@ -392,7 +396,7 @@ class networkInterfaces():
 
     def _is_keyword(self, str):
         # The additional split here is for allow- keyword
-        if (str in self.network_elems.keys() or
+        if (str in list(self.network_elems.keys()) or
             str.split('-')[0] == 'allow'):
             return 1
         return 0
@@ -403,7 +407,7 @@ class networkInterfaces():
 
     def get_allow_classes_for_iface(self, ifacename):
         classes = []
-        for class_name, ifacenames in self.allow_classes.items():
+        for class_name, ifacenames in list(self.allow_classes.items()):
             if ifacename in ifacenames:
                 classes.append(class_name)
         return classes
@@ -451,12 +455,17 @@ class networkInterfaces():
                     self._currentfile_has_template = False
                 else:
                     self._currentfile_has_template = True
-            except Exception, e:
+            except Exception as e:
                 self._parse_error(self._currentfile, -1,
                                   'failed to render template (%s). Continue without template rendering ...'
                                   % str(e))
                 rendered_filedata = None
             if rendered_filedata:
+
+                if isinstance(rendered_filedata, bytes):
+                    # some template engine might return bytes but we want str
+                    rendered_filedata = rendered_filedata.decode()
+
                 self.process_interfaces(rendered_filedata)
                 return
         self.process_interfaces(filedata)
@@ -470,8 +479,8 @@ class networkInterfaces():
         try:
             with open(filename) as f:
                 filedata = f.read()
-        except Exception, e:
-            self.logger.warn('error processing file %s (%s)',
+        except Exception as e:
+            self.logger.warning('error processing file %s (%s)',
                              filename, str(e))
             return
         self.read_filedata(filedata)
@@ -508,8 +517,8 @@ class networkInterfaces():
         parser arguments
         """
         if not self.interfacesfile and not self.interfacesfileiobuf:
-            self.logger.warn('no terminal line stdin used or ')
-            self.logger.warn('no network interfaces file defined.')
+            self.logger.warning('no terminal line stdin used or ')
+            self.logger.warning('no network interfaces file defined.')
             self.warns += 1
             return
 
