@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2015-2017 Cumulus Networks, Inc. All rights reserved.
 #
@@ -40,6 +40,8 @@ class policymanager():
         self.logger = logging.getLogger('ifupdown.' +
                             self.__class__.__name__)
 
+        self.logger.info("policymanager init")
+
         # we grab the json files from a known location and make sure that
         # the defaults_policy is checked first
         user_files = glob.glob('/etc/network/ifupdown2/policy.d/*.json')
@@ -54,17 +56,26 @@ class policymanager():
                     system_array = json.load(fd)
                 self.logger.debug('reading %s system policy defaults config' \
                                   % filename)
-            except Exception, e:
+            except Exception as e:
                 self.logger.warning('could not read %s system policy defaults config' \
                                   % filename)
                 self.logger.warning('    exception is %s' % str(e))
 
-            for module in system_array.keys():
-                if self.system_policy_array.has_key(module):
+            for module in list(system_array.keys()):
+                if module in self.system_policy_array:
                     self.logger.debug("policymanager: merging system module %s policy with file %s" % (module, filename))
                     self.system_policy_array[module].update(system_array[module])
                 else:
-                    self.system_policy_array[module] = system_array[module]
+                    json_dict = system_array[module]
+
+                    if isinstance(json_dict, dict):
+                        self.system_policy_array[module] = system_array[module]
+                    elif module != "README":
+                        self.logger.warning(
+                            "file %s contains an invalid policy schema, key "
+                            "\"%s\" contains %s when a dictionary is expected" %
+                            (filename, module, type(json_dict))
+                        )
 
         # take care of user defined policy defaults
         self.user_policy_array = {}
@@ -75,13 +86,13 @@ class policymanager():
                     user_array = json.load(fd)
                 self.logger.debug('reading %s policy user defaults config' \
                                   % filename)
-            except Exception, e:
+            except Exception as e:
                 self.logger.warning('could not read %s user policy defaults config' \
                                   % filename)
                 self.logger.warning('    exception is %s' % str(e))
             # customer added module attributes
-            for module in user_array.keys():
-                if self.system_policy_array.has_key(module):
+            for module in list(user_array.keys()):
+                if module in self.system_policy_array:
                     # warn user that we are overriding the system module setting
                     self.logger.debug('warning: overwriting system with user module %s from file %s' \
                                       % (module,filename))
