@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -10,26 +10,22 @@
 import os
 import sys
 
-from sets import Set
-
 try:
     from ifupdown2.ifupdown.graph import *
-    from ifupdown2.ifupdown.ifupdownbase import *
-
     from ifupdown2.ifupdown.iface import *
     from ifupdown2.ifupdown.utils import utils
     from ifupdown2.ifupdown.statemanager import *
 
+    import ifupdown2.ifupdown.policymanager as policymanager
     import ifupdown2.ifupdown.ifupdownflags as ifupdownflags
-except ImportError:
+except (ImportError, ModuleNotFoundError):
     from ifupdown.graph import *
-    from ifupdown.ifupdownbase import *
-
     from ifupdown.iface import *
     from ifupdown.utils import utils
     from ifupdown.statemanager import *
 
     import ifupdown.ifupdownflags as ifupdownflags
+    import ifupdown.policymanager as policymanager
 
 
 class ifaceSchedulerFlags():
@@ -50,6 +46,11 @@ class ifaceScheduler():
 
     _SCHED_STATUS = True
 
+    VRF_MGMT_DEVNAME = policymanager.policymanager_api.get_module_globals(
+        module_name="vrf",
+        attr="vrf-mgmt-devname"
+    )
+
     @classmethod
     def reset(cls):
         cls._STATE_CHECK = True
@@ -100,10 +101,13 @@ class ifaceScheduler():
                         ifupdownobj.logger.debug(msg)
                         m.run(ifaceobj, op,
                               ifaceobj_getfunc=ifupdownobj.get_ifaceobjs)
-            except Exception, e:
+            except Exception as e:
                 if not ifupdownobj.ignore_error(str(e)):
-                   err = 1
-                   ifupdownobj.logger.error(str(e))
+                    err = 1
+                    #import traceback
+                    #traceback.print_exc()
+
+                    ifupdownobj.logger.error(str(e))
                 # Continue with rest of the modules
                 pass
             finally:
@@ -132,7 +136,7 @@ class ifaceScheduler():
                     %(ifacename, op, mname))
                 try:
                     utils.exec_command(mname, env=cenv)
-                except Exception, e:
+                except Exception as e:
                     if "permission denied" in str(e).lower():
                         ifupdownobj.logger.warning('%s: %s %s' % (ifacename, op, str(e)))
                     else:
@@ -148,7 +152,6 @@ class ifaceScheduler():
         # if interface exists in the system
         ifacename = ifaceobjs[0].name
         ifupdownobj.logger.info('%s: running ops ...' %ifacename)
-
         if ('down' in ops[0] and
                 ifaceobjs[0].type != ifaceType.BRIDGE_VLAN and
                 ifaceobjs[0].addr_method != 'ppp' and
@@ -169,9 +172,9 @@ class ifaceScheduler():
             if handler:
                 try:
                     handler(ifupdownobj, ifaceobjs[0])
-                except Exception, e:
+                except Exception as e:
                     if not ifupdownobj.link_master_slave_ignore_error(str(e)):
-                       ifupdownobj.logger.warn('%s: %s'
+                       ifupdownobj.logger.warning('%s: %s'
                                    %(ifaceobjs[0].name, str(e)))
                     pass
             for ifaceobj in ifaceobjs:
@@ -184,8 +187,8 @@ class ifaceScheduler():
             try:
                 [posthookfunc(ifupdownobj, ifaceobj, ops[0])
                     for ifaceobj in ifaceobjs]
-            except Exception, e:
-                ifupdownobj.logger.warn('%s' %str(e))
+            except Exception as e:
+                ifupdownobj.logger.warning('%s' %str(e))
                 pass
 
     @classmethod
@@ -230,7 +233,7 @@ class ifaceScheduler():
                         ifupdownobj.logger.info('%s: skipping interface down,'
                             %ifaceobj.name + ' upperiface %s still around ' %u)
                     else:
-                        ifupdownobj.logger.warn('%s: skipping interface down,'
+                        ifupdownobj.logger.warning('%s: skipping interface down,'
                             %ifaceobj.name + ' upperiface %s still around ' %u)
                 return False
         return True
@@ -265,6 +268,14 @@ class ifaceScheduler():
             # Run lowerifaces or dependents
             dlist = ifaceobj.lowerifaces
             if dlist:
+
+                if ifaceobj.link_kind == ifaceLinkKind.VRF:
+                    # remove non-auto lowerifaces from 'dlist'
+                    for lower_ifname in list(dlist):
+                        for lower_ifaceobj in ifupdownobj.get_ifaceobjs(lower_ifname) or []:
+                            if lower_ifaceobj and not lower_ifaceobj.auto and ifaceobj.name == cls.VRF_MGMT_DEVNAME:
+                                dlist.remove(lower_ifname)
+
                 ifupdownobj.logger.debug('%s: found dependents %s'
                             %(ifacename, str(dlist)))
                 try:
@@ -285,7 +296,7 @@ class ifaceScheduler():
                                             ifacename, order,
                                             followdependents,
                                             continueonfailure=False)
-                except Exception, e:
+                except Exception as e:
                     if (ifupdownobj.ignore_error(str(e))):
                         pass
                     else:
@@ -306,7 +317,7 @@ class ifaceScheduler():
             try:
               cls.run_iface_graph(ifupdownobj, ifacename, ops, parent,
                       order, followdependents)
-            except Exception, e:
+            except Exception as e:
                 if continueonfailure:
                     if ifupdownobj.logger.isEnabledFor(logging.DEBUG):
                         traceback.print_tb(sys.exc_info()[2])
@@ -347,7 +358,7 @@ class ifaceScheduler():
                                             ifacename,
                                             followdependents,
                                             continueonfailure=True)
-                except Exception, e:
+                except Exception as e:
                     if (ifupdownobj.ignore_error(str(e))):
                         pass
                     else:
@@ -363,10 +374,10 @@ class ifaceScheduler():
             try:
               cls.run_iface_graph_upper(ifupdownobj, ifacename, ops, parent,
                       followdependents, skip_root)
-            except Exception, e:
+            except Exception as e:
                 if ifupdownobj.logger.isEnabledFor(logging.DEBUG):
                     traceback.print_tb(sys.exc_info()[2])
-                ifupdownobj.logger.warn('%s : %s' %(ifacename, str(e)))
+                ifupdownobj.logger.warning('%s : %s' %(ifacename, str(e)))
                 pass
 
     @classmethod
@@ -389,7 +400,7 @@ class ifaceScheduler():
             ifaceobj = ifupdownobj.get_ifaceobj_first(ifacename)
             if not ifaceobj:
                continue
-            ulist = Set(ifaceobj.upperifaces).difference(upperifacenames)
+            ulist = set(ifaceobj.upperifaces or []).difference(upperifacenames)
             nulist = []
             for u in ulist:
                 uifaceobj = ifupdownobj.get_ifaceobj_first(u)
@@ -430,9 +441,9 @@ class ifaceScheduler():
                 if not ifaceobjs:
                    continue
                 cls.run_iface_list_ops(ifupdownobj, ifaceobjs, ops)
-            except Exception, e:
+            except Exception as e:
                 if continueonfailure:
-                    ifupdownobj.logger.warn('%s' %str(e))
+                    ifupdownobj.logger.warning('%s' %str(e))
 
     @classmethod
     def _dump_dependency_info(cls, ifupdownobj, ifacenames,
@@ -440,16 +451,16 @@ class ifaceScheduler():
         ifupdownobj.logger.info('{\n')
         ifupdownobj.logger.info('\nifaceobjs:')
         for iname in ifacenames:
-	        iobjs = ifupdownobj.get_ifaceobjs(iname)
-	        for iobj in iobjs:
-		        iobj.dump(ifupdownobj.logger)
+            iobjs = ifupdownobj.get_ifaceobjs(iname)
+            for iobj in iobjs:
+                iobj.dump(ifupdownobj.logger)
         if (dependency_graph):
             ifupdownobj.logger.info('\nDependency Graph:')
             ifupdownobj.logger.info(dependency_graph)
-	    if (indegrees):
-		    ifupdownobj.logger.info('\nIndegrees:')
-		    ifupdownobj.logger.info(indegrees)
-	    ifupdownobj.logger.info('}\n')
+        if (indegrees):
+            ifupdownobj.logger.info('\nIndegrees:')
+            ifupdownobj.logger.info(indegrees)
+        ifupdownobj.logger.info('}\n')
 
     @classmethod
     def get_sorted_iface_list(cls, ifupdownobj, ifacenames, ops,
@@ -459,7 +470,7 @@ class ifaceScheduler():
         # Get a sorted list of all interfaces
         if not indegrees:
             indegrees = OrderedDict()
-            for ifacename in dependency_graph.keys():
+            for ifacename in list(dependency_graph.keys()):
                 indegrees[ifacename] = ifupdownobj.get_iface_refcnt(ifacename)
 
         #cls._dump_dependency_info(ifupdownobj, ifacenames,
@@ -522,7 +533,7 @@ class ifaceScheduler():
         skip_ifacesort = int(ifupdownobj.config.get('skip_ifacesort', '0'))
         if not skip_ifacesort and not indegrees:
             indegrees = OrderedDict()
-            for ifacename in dependency_graph.keys():
+            for ifacename in list(dependency_graph.keys()):
                 indegrees[ifacename] = ifupdownobj.get_iface_refcnt(ifacename)
 
         if not ifupdownflags.flags.ALL:
@@ -560,7 +571,7 @@ class ifaceScheduler():
                     ifupdownobj.logger.debug('graph roots (interfaces that ' +
                             'dont have dependents):' + ' %s' %str(run_queue))
                 else:
-                    ifupdownobj.logger.warn('interface sort returned None')
+                    ifupdownobj.logger.warning('interface sort returned None')
 
         # If queue not present, just run interfaces that were asked by the
         # user
