@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2014-2017 Cumulus Networks, Inc. All rights reserved.
 # Author: Roopa Prabhu, roopa@cumulusnetworks.com
@@ -8,6 +8,7 @@ import os
 import re
 import logging
 import traceback
+from functools import reduce
 
 try:
     from ifupdown2.ifupdown.iface import *
@@ -16,7 +17,7 @@ try:
     import ifupdown2.ifupdown.exceptions as exceptions
     import ifupdown2.ifupdown.policymanager as policymanager
     import ifupdown2.ifupdown.ifupdownflags as ifupdownflags
-except ImportError:
+except (ImportError, ModuleNotFoundError):
     from ifupdown.iface import *
     from ifupdown.utils import utils
 
@@ -67,7 +68,7 @@ class moduleBase(object):
         error_msg = 'this attribute doesn\'t exist or isn\'t supported'
 
         # first check module_defaults
-        for key, value in policymanager.policymanager_api.get_module_defaults(self.modulename).items():
+        for key, value in list(policymanager.policymanager_api.get_module_defaults(self.modulename).items()):
             if not key in attrs:
                 self.logger.warning('%s: %s: %s' % (self.modulename, key, error_msg))
                 continue
@@ -79,7 +80,7 @@ class moduleBase(object):
             policy_attrs = policy_modinfo.get('attrs', {})
             update_attrs = dict()
 
-            for attr_name, attr_description in policy_attrs.items():
+            for attr_name, attr_description in list(policy_attrs.items()):
                 if attr_name not in attrs:
                     self.logger.warning('%s: %s: %s' % (self.modulename, attr_name, error_msg))
                 else:
@@ -95,7 +96,7 @@ class moduleBase(object):
             if self.logger.getEffectiveLevel() == logging.DEBUG:
                 traceback.print_stack()
                 traceback.print_exc()
-            self.logger.warn(str)
+            self.logger.warning(str)
             if ifaceobj:
                 ifaceobj.set_status(ifaceStatus.WARNING)
         pass
@@ -119,7 +120,7 @@ class moduleBase(object):
         try:
             utils.exec_command('%s -x %s' %
                                (utils.pidof_cmd, procName))
-        except:
+        except Exception:
             return False
         else:
             return True
@@ -131,21 +132,21 @@ class moduleBase(object):
                 lines = f.readlines()
                 for line in lines[2:]:
                     ifacenames.append(line.split()[0].strip(': '))
-            except:
+            except Exception:
                 raise
         return ifacenames
 
     def parse_regex(self, ifacename, expr, ifacenames=None):
         try:
             proc_ifacenames = self.get_ifaces_from_proc()
-        except:
-            self.logger.warn('%s: error reading ifaces from proc' %ifacename)
+        except Exception:
+            self.logger.warning('%s: error reading ifaces from proc' %ifacename)
 
         for proc_ifacename in proc_ifacenames:
             try:
                 if re.search(expr + '$', proc_ifacename):
                     yield proc_ifacename
-            except Exception, e:
+            except Exception as e:
                 raise Exception('%s: error searching regex \'%s\' in %s (%s)'
                                 %(ifacename, expr, proc_ifacename, str(e)))
         if not ifacenames:
@@ -154,7 +155,7 @@ class moduleBase(object):
             try:
                 if re.search(expr + '$', ifacename):
                     yield ifacename
-            except Exception, e:
+            except Exception as e:
                 raise Exception('%s: error searching regex \'%s\' in %s (%s)'
                                 %(ifacename, expr, ifacename, str(e)))
 
@@ -174,7 +175,7 @@ class moduleBase(object):
                     '  or swp[1-10]sub[0-4].300')
 
         if ',' in expr:
-            self.logger.warn('%s: comma are not supported in glob: %s' % (ifacename, errmsg))
+            self.logger.warning('%s: comma are not supported in glob: %s' % (ifacename, errmsg))
             yield expr
             return
 
@@ -221,7 +222,7 @@ class moduleBase(object):
 
         else:
             # Could not match anything.
-            self.logger.warn('%s: %s' %(ifacename, errmsg))
+            self.logger.warning('%s: %s' %(ifacename, errmsg))
             yield expr
 
     def parse_port_list(self, ifacename, port_expr, ifacenames=None):
@@ -279,8 +280,8 @@ class moduleBase(object):
                 return 0
             with open(filename, 'w') as f:
                 f.write(strexpr)
-        except IOError, e:
-            self.logger.warn('error writing to file %s'
+        except IOError as e:
+            self.logger.warning('error writing to file %s'
                 %filename + '(' + str(e) + ')')
             return -1
         return 0
@@ -291,7 +292,7 @@ class moduleBase(object):
             self.logger.info('reading \'%s\'' %filename)
             with open(filename, 'r') as f:
                 return f.readlines()
-        except:
+        except Exception:
             return None
         return None
 
@@ -301,7 +302,7 @@ class moduleBase(object):
             self.logger.info('reading \'%s\'' %filename)
             with open(filename, 'r') as f:
                 return f.readline().strip('\n')
-        except:
+        except Exception:
             return None
         return None
 
@@ -324,7 +325,7 @@ class moduleBase(object):
             return self._bridge_stp_user_space
         try:
             self._bridge_stp_user_space = self.sysctl_get('net.bridge.bridge-stp-user-space')
-        except:
+        except Exception:
             self._bridge_stp_user_space = 0
 
         return self._bridge_stp_user_space
@@ -371,21 +372,21 @@ class moduleBase(object):
         try:
             retattrs = []
             attrsdict = self._modinfo.get('attrs')
-            for attrname, attrvals in attrsdict.iteritems():
+            for attrname, attrvals in attrsdict.items():
                 if not attrvals or attrvals.get('deprecated'):
                     continue
                 retattrs.append(attrname)
                 if 'aliases' in attrvals:
                     retattrs.extend(attrvals['aliases'])
             return retattrs
-        except:
+        except Exception:
             return None
 
     def get_mod_attr(self, attrname):
         """ returns module attr info """
         try:
             return self._modinfo.get('attrs', {}).get(attrname)
-        except:
+        except Exception:
             return None
 
     def get_mod_subattr(self, attrname, subattrname):
@@ -393,14 +394,14 @@ class moduleBase(object):
         try:
             return reduce(lambda d, k: d[k], ['attrs', attrname, subattrname],
                          self._modinfo)
-        except:
+        except Exception:
             return None
 
     def get_modinfo(self):
         """ return module info """
         try:
             return self._modinfo
-        except:
+        except Exception:
             return {}
 
     def get_attr_default_value(self, attrname):
@@ -410,7 +411,7 @@ class moduleBase(object):
         """ return the ifupdown scripts replaced by the current module """
         try:
             return self.overrides_ifupdown_scripts
-        except:
+        except Exception:
             return []
 
     def _get_reserved_vlan_range(self):
@@ -422,12 +423,23 @@ class moduleBase(object):
             (s, e) = utils.exec_command(get_resvvlan).strip('\n').split('-')
             start = int(s)
             end = int(e)
-        except Exception, e:
+        except Exception as e:
             self.logger.debug('%s failed (%s)' %(get_resvvlan, str(e)))
             # ignore errors
             pass
         return (start, end)
 
+    def _get_vrf_context(self):
+        vrfid = 'default'
+        try:
+            vrfid = utils.exec_command('/usr/sbin/ip vrf id').strip()
+        except Exception as e:
+            self.logger.debug('failed to get vrf id (%s)' %str(e))
+            # ignore errors
+            vrfid = None
+            pass
+        return vrfid
+
     def _handle_reserved_vlan(self, vlanid, logprefix='', end=-1):
         """ Helper function to check and warn if the vlanid falls in the
         reserved vlan range """
@@ -470,7 +482,7 @@ class moduleBase(object):
             return -1
         try:
             vid = int(vid_str)
-        except:
+        except Exception:
             return -1
         return vid
 
@@ -487,7 +499,7 @@ class moduleBase(object):
         vid_str = ifaceobj.get_attr_value_first('vlan-id')
         try:
             if vid_str: return int(vid_str)
-        except:
+        except Exception:
             return -1
 
         return self._get_vlan_id_from_ifacename(ifaceobj.name)
