@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
-# Copyright (C) 2015, 2017 Cumulus Networks, Inc. all rights reserved
+# Copyright (C) 2015-2020 Cumulus Networks, Inc. all rights reserved
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -26,8 +26,7 @@
 #
 
 from collections import OrderedDict
-from ipaddr import IPv4Address, IPv6Address
-from nlpacket import *
+from .nlpacket import *
 from select import select
 from struct import pack, unpack
 import logging
@@ -58,7 +57,7 @@ class Sequence(object):
     def __init__(self):
         self._next = 0
 
-    def next(self):
+    def __next__(self):
         self._next += 1
         return self._next
 
@@ -130,7 +129,10 @@ class NetlinkManager(object):
         self._debug_set_clear((RTM_NEWROUTE, RTM_DELROUTE, RTM_GETROUTE), enabled)
 
     def debug_netconf(self, enabled):
-        self._debug_set_clear((RTM_GETNETCONF, RTM_NEWNETCONF), enabled)
+        self._debug_set_clear((RTM_GETNETCONF, RTM_NEWNETCONF, RTM_DELNETCONF), enabled)
+
+    def debug_mdb(self, enabled):
+        self._debug_set_clear((RTM_GETMDB, RTM_NEWMDB, RTM_DELMDB), enabled)
 
     def debug_this_packet(self, mtype):
         if mtype in self.debug:
@@ -142,8 +144,39 @@ class NetlinkManager(object):
         The TX socket is used for install requests, sending RTM_GETXXXX
         requests, etc
         """
-        self.tx_socket = socket.socket(socket.AF_NETLINK, socket.SOCK_RAW, 0)
-        self.tx_socket.bind((self.pid, 0))
+        try:
+            self.tx_socket = socket.socket(socket.AF_NETLINK, socket.SOCK_RAW, 0)
+
+            # bind retry mechanism:
+            # in some cases we are running into weird issues... Address already in use
+            # to counter this problem, we will retry up to NLMANAGER_BIND_RETRY times to
+            # bind our socket, every time increasing the address (or pid) that we bind it
+            # to. NLMANAGER_BIND_RETRY default to 4242
+            for i in range(0, int(os.getenv("NLMANAGER_BIND_RETRY", 4242))):
+                try:
+                    self.tx_socket.bind((self.pid + i, 0))
+                    # the bind call succeeded, we need to update self.pid
+                    # to reflect the correct value we are binded to. If we
+                    # couldn't bind to our real pid (os.getpid()) warn user
+                    # to avoid confusion (via debug logs).
+                    if i != 0:
+                        log.debug(
+                            "nlmanager: pid %s already in use - binding netlink socket to pid %s"
+                            % (self.pid, self.pid + i)
+                        )
+                    self.pid = self.pid + i
+                    return
+                except Exception:
+                    pass
+            # if we reach this code it means all our bind calls failed. We are trying to
+            # bind the socket one last time on the original parameters if not we will not
+            # be catching the exception
+            self.tx_socket.bind((self.pid, 0))
+        except Exception:
+            if self.tx_socket:
+                self.tx_socket.close()
+                self.tx_socket = None
+            raise
 
     def tx_nlpacket_raw(self, message):
         """
@@ -322,6 +355,9 @@ class NetlinkManager(object):
                         elif msgtype in (RTM_GETNETCONF, RTM_NEWNETCONF):
                             msg = Netconf(msgtype, nlpacket.debug, use_color=self.use_color)
 
+                        elif msgtype in (RTM_GETMDB, RTM_NEWMDB, RTM_DELMDB):
+                            msg = MDB(msgtype, nlpacket.debug, use_color=self.use_color)
+
                         else:
                             raise Exception("RXed unknown netlink message type %s" % msgtype)
 
@@ -334,14 +370,12 @@ class NetlinkManager(object):
                     data = data[length:]
 
     def ip_to_afi(self, ip):
-        type_ip = type(ip)
-
-        if type_ip == IPv4Address:
+        if ip.version == 4:
             return socket.AF_INET
-        elif type_ip == IPv6Address:
+        elif ip.version == 6:
             return socket.AF_INET6
         else:
-            raise Exception("%s is an invalid IP type" % type_ip)
+            raise Exception("%s is an invalid IP type" % type(ip))
 
     def request_dump(self, rtm_type, family, debug):
         """
@@ -365,13 +399,17 @@ class NetlinkManager(object):
             msg = Route(rtm_type, debug, use_color=self.use_color)
             msg.body = pack('Bxxxii', family, 0, 0)
 
+        elif rtm_type == RTM_GETMDB:
+            msg = MDB(rtm_type, debug, use_color=self.use_color)
+            msg.body = pack('Bxxxii', family, 0, 0)
+
         else:
             log.error("request_dump RTM_GET %s is not supported" % rtm_type)
             return None
 
         msg.flags = NLM_F_REQUEST | NLM_F_DUMP
         msg.attributes = {}
-        msg.build_message(self.sequence.next(), self.pid)
+        msg.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(msg)
 
     # ======
@@ -417,7 +455,7 @@ class NetlinkManager(object):
                 if nexthop:
                     route.add_attribute(Route.RTA_GATEWAY, nexthop)
                 route.add_attribute(Route.RTA_OIF, interface_index)
-                route.build_message(self.sequence.next(), self.pid)
+                route.build_message(next(self.sequence), self.pid)
                 total_message = tx_or_concat_message(total_message, route)
 
             if total_message:
@@ -425,7 +463,7 @@ class NetlinkManager(object):
 
         if ecmp_routes:
 
-            for (route_key, value) in ecmp_routes.iteritems():
+            for (route_key, value) in ecmp_routes.items():
                 (afi, ip, mask) = route_key
 
                 route = Route(rtm_command, debug, use_color=self.use_color)
@@ -435,7 +473,7 @@ class NetlinkManager(object):
                 route.family = afi
                 route.add_attribute(Route.RTA_DST, ip)
                 route.add_attribute(Route.RTA_MULTIPATH, value)
-                route.build_message(self.sequence.next(), self.pid)
+                route.build_message(next(self.sequence), self.pid)
                 total_message = tx_or_concat_message(total_message, route)
 
             if total_message:
@@ -457,9 +495,7 @@ class NetlinkManager(object):
 
     def route_get(self, ip, debug=False):
         """
-        ip must be one of the following:
-        - IPv4Address
-        - IPv6Address
+        ip must be ipnetwork.IPNetwork
         """
         # Transmit a RTM_GETROUTE to query for the route we want
         route = Route(RTM_GETROUTE, debug, use_color=self.use_color)
@@ -470,7 +506,7 @@ class NetlinkManager(object):
         route.body = pack('Bxxxxxxxi', afi, 0)
         route.family = afi
         route.add_attribute(Route.RTA_DST, ip)
-        route.build_message(self.sequence.next(), self.pid)
+        route.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(route)
 
     def routes_dump(self, family=socket.AF_UNSPEC, debug=True):
@@ -480,7 +516,7 @@ class NetlinkManager(object):
         """
         Print a table of 'routes'
         """
-        print "Prefix            Nexthop           ifindex"
+        print("Prefix            Nexthop           ifindex")
 
         for x in routes:
             if Route.RTA_DST not in x.attributes:
@@ -488,10 +524,10 @@ class NetlinkManager(object):
                 continue
 
             ip = "%s/%d" % (x.attributes[Route.RTA_DST].value, x.src_len)
-            print "%-15s   %-15s   %s" %\
+            print("%-15s   %-15s   %s" %\
                 (ip,
                  str(x.attributes[Route.RTA_GATEWAY].value) if Route.RTA_GATEWAY in x.attributes else None,
-                 x.attributes[Route.RTA_OIF].value)
+                 x.attributes[Route.RTA_OIF].value))
 
     # =====
     # Links
@@ -506,7 +542,7 @@ class NetlinkManager(object):
         link.flags = NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('=Bxxxiii', socket.AF_UNSPEC, 0, 0, 0)
         link.add_attribute(Link.IFLA_IFNAME, ifname)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
 
         try:
             return self.tx_nlpacket_get_response(link)[0]
@@ -524,7 +560,7 @@ class NetlinkManager(object):
         link = Link(RTM_GETLINK, debug, use_color=self.use_color)
         link.flags = NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('=Bxxxiii', socket.AF_UNSPEC, ifindex, 0, 0)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         try:
             return self.tx_nlpacket_get_response(link)[0]
         except NetlinkNoAddressError:
@@ -559,7 +595,7 @@ class NetlinkManager(object):
         else:
             msg.flags |= NLM_F_DUMP
 
-        msg.build_message(self.sequence.next(), self.pid)
+        msg.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(msg)
 
     def link_set_attrs(self, ifname, kind=None, slave_kind=None, ifindex=0, ifla={}, ifla_info_data={}, ifla_info_slave_data={}):
@@ -569,7 +605,7 @@ class NetlinkManager(object):
         link.flags = NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('Bxxxiii', socket.AF_UNSPEC, ifindex, 0, 0)
 
-        for nl_attr, value in ifla.items():
+        for nl_attr, value in list(ifla.items()):
             link.add_attribute(nl_attr, value)
 
         if ifname:
@@ -585,7 +621,7 @@ class NetlinkManager(object):
             linkinfo[Link.IFLA_INFO_SLAVE_DATA] = ifla_info_slave_data
 
         link.add_attribute(Link.IFLA_LINKINFO, linkinfo)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_add_set(self, kind,
@@ -604,7 +640,7 @@ class NetlinkManager(object):
         link.flags = NLM_F_CREATE | NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('Bxxxiii', socket.AF_UNSPEC, ifindex, 0, 0)
 
-        for nl_attr, value in ifla.items():
+        for nl_attr, value in list(ifla.items()):
             link.add_attribute(nl_attr, value)
 
         if ifname:
@@ -619,7 +655,7 @@ class NetlinkManager(object):
             linkinfo[Link.IFLA_INFO_SLAVE_DATA] = ifla_info_slave_data
         link.add_attribute(Link.IFLA_LINKINFO, linkinfo)
 
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_del(self, ifindex=None, ifname=None):
@@ -634,7 +670,7 @@ class NetlinkManager(object):
         link = Link(RTM_DELLINK, debug, use_color=self.use_color)
         link.flags = NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('Bxxxiii', socket.AF_UNSPEC, ifindex, 0, 0)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def _link_add(self, ifindex, ifname, kind, ifla_info_data, mtu=None):
@@ -658,7 +694,7 @@ class NetlinkManager(object):
             Link.IFLA_INFO_KIND: kind,
             Link.IFLA_INFO_DATA: ifla_info_data
         })
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_add_bridge(self, ifname, ifla_info_data={}, mtu=None):
@@ -692,24 +728,24 @@ class NetlinkManager(object):
 
         return self._link_add(ifindex, ifname, 'vlan', ifla_info_data)
 
-    def link_add_macvlan(self, ifindex, ifname):
+    def link_add_macvlan(self, ifindex, ifname, macvlan_mode):
         """
         ifindex is the index of the parent interface that this sub-interface
         is being added to
         """
-        return self._link_add(ifindex, ifname, 'macvlan', {Link.IFLA_MACVLAN_MODE: Link.MACVLAN_MODE_PRIVATE})
-
-    def link_add_xfrm(self, physdev, xfrm_ifname, xfrm_id):
-        """
-        ifindex is the index of the parent interface that this sub-interface
-        is being added to
-        """
-        ifla_info_data = {
-            Link.IFLA_XFRM_IF_ID: int(xfrm_id),
-            Link.IFLA_XFRM_LINK: int(physdev)
-        }
-        
-        return self._link_add(ifindex=None, ifname=xfrm_ifname, kind='xfrm', ifla_info_data=ifla_info_data)
+        return self._link_add(
+            ifindex,
+            ifname,
+            'macvlan',
+            {
+                Link.IFLA_MACVLAN_MODE: {
+                    "private": Link.MACVLAN_MODE_PRIVATE,
+                    "vepa": Link.MACVLAN_MODE_VEPA,
+                    "bridge": Link.MACVLAN_MODE_BRIDGE,
+                    "passthru": Link.MACVLAN_MODE_PASSTHRU
+                }.get(macvlan_mode, Link.MACVLAN_MODE_PRIVATE)
+            }
+        )
 
     def vlan_get(self, filter_ifindex=None, filter_vlanid=None, compress_vlans=True):
         """
@@ -728,7 +764,7 @@ class NetlinkManager(object):
         else:
             link.add_attribute(Link.IFLA_EXT_MASK, Link.RTEXT_FILTER_BRVLAN)
 
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         reply = self.tx_nlpacket_get_response(link)
 
         iface_vlans = {}
@@ -758,7 +794,7 @@ class NetlinkManager(object):
               25: 0x08000200  ....  Nested Attribute - Length 0x0008 (8),  Type 0x0002 (2) IFLA_BRIDGE_VLAN_INFO
               26: 0x00001400  ....
             '''
-            for (x_type, x_value) in ifla_af_spec.iteritems():
+            for (x_type, x_value) in ifla_af_spec.items():
                 if x_type == Link.IFLA_BRIDGE_VLAN_INFO:
                     for (vlan_flag, vlan_id) in x_value:
                         if filter_vlanid is None or vlan_id in filter_vlanid:
@@ -789,10 +825,10 @@ class NetlinkManager(object):
         range_begin_vlan_id = None
         range_flag = None
 
-        print "   Interface  VLAN  Flags"
-        print "  ==========  ====  ====="
+        print("   Interface  VLAN  Flags")
+        print("  ==========  ====  =====")
 
-        for (ifname, vlan_tuples) in sorted(iface_vlans.iteritems()):
+        for (ifname, vlan_tuples) in sorted(iface_vlans.items()):
             for (vlan_id, vlan_flag) in sorted(vlan_tuples):
 
                 if vlan_flag & Link.BRIDGE_VLAN_INFO_RANGE_BEGIN:
@@ -806,15 +842,15 @@ class NetlinkManager(object):
                         log.warning("BRIDGE_VLAN_INFO_RANGE_END is %d but we never saw a BRIDGE_VLAN_INFO_RANGE_BEGIN" % vlan_id)
                         range_begin_vlan_id = vlan_id
 
-                    for x in xrange(range_begin_vlan_id, vlan_id + 1):
-                        print "  %10s  %4d  %s" % (ifname, x, vlan_flag_to_string(vlan_flag))
+                    for x in range(range_begin_vlan_id, vlan_id + 1):
+                        print("  %10s  %4d  %s" % (ifname, x, vlan_flag_to_string(vlan_flag)))
                         ifname = ''
 
                     range_begin_vlan_id = None
                     range_flag = None
 
                 else:
-                    print "  %10s  %4d  %s" % (ifname, vlan_id, vlan_flag_to_string(vlan_flag))
+                    print("  %10s  %4d  %s" % (ifname, vlan_id, vlan_flag_to_string(vlan_flag)))
                     ifname = ''
 
 
@@ -868,7 +904,7 @@ class NetlinkManager(object):
             ]
 
         link.add_attribute(Link.IFLA_AF_SPEC, ifla_af_spec)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_add_bridge_vlan(self, ifindex, vlanid_start, vlanid_end=None, pvid=False, untagged=False, master=False):
@@ -904,7 +940,7 @@ class NetlinkManager(object):
         link.flags = NLM_F_REQUEST | NLM_F_ACK
         link.body = pack('=BxxxiLL', socket.AF_UNSPEC, 0, if_flags, if_change)
         link.add_attribute(Link.IFLA_IFNAME, ifname)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_set_protodown(self, ifname, state):
@@ -921,7 +957,7 @@ class NetlinkManager(object):
         link.body = pack('=BxxxiLL', socket.AF_UNSPEC, 0, 0, 0)
         link.add_attribute(Link.IFLA_IFNAME, ifname)
         link.add_attribute(Link.IFLA_PROTO_DOWN, protodown)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     def link_set_master(self, ifname, master_ifindex=0, state=None):
@@ -946,7 +982,7 @@ class NetlinkManager(object):
         link.body = pack('=BxxxiLL', socket.AF_UNSPEC, 0, if_flags, if_change)
         link.add_attribute(Link.IFLA_IFNAME, ifname)
         link.add_attribute(Link.IFLA_MASTER, master_ifindex)
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     # =========
@@ -962,7 +998,7 @@ class NetlinkManager(object):
         nbr.body = pack('=BxxxiHBB', afi, ifindex, Neighbor.NUD_REACHABLE, service_hdr_flags, Route.RTN_UNICAST)
         nbr.add_attribute(Neighbor.NDA_DST, ip)
         nbr.add_attribute(Neighbor.NDA_LLADDR, mac)
-        nbr.build_message(self.sequence.next(), self.pid)
+        nbr.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(nbr)
 
     def neighbor_del(self, afi, ifindex, ip, mac):
@@ -975,7 +1011,7 @@ class NetlinkManager(object):
         nbr.body = pack('=BxxxiHBB', afi, ifindex, Neighbor.NUD_REACHABLE, service_hdr_flags, Route.RTN_UNICAST)
         nbr.add_attribute(Neighbor.NDA_DST, ip)
         nbr.add_attribute(Neighbor.NDA_LLADDR, mac)
-        nbr.build_message(self.sequence.next(), self.pid)
+        nbr.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(nbr)
 
     def link_add_vxlan(self, ifname, vxlanid, dstport=None, local=None,
@@ -1009,7 +1045,7 @@ class NetlinkManager(object):
             Link.IFLA_INFO_DATA: info_data
         })
 
-        link.build_message(self.sequence.next(), self.pid)
+        link.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(link)
 
     # =========
@@ -1027,7 +1063,7 @@ class NetlinkManager(object):
         msg.body = pack('=Bxxxi', socket.AF_UNSPEC, 0)
         msg.flags = NLM_F_REQUEST | NLM_F_ACK | NLM_F_DUMP
 
-        msg.build_message(self.sequence.next(), self.pid)
+        msg.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(msg)
 
     # =======
@@ -1044,5 +1080,16 @@ class NetlinkManager(object):
         msg = Netconf(RTM_GETNETCONF, debug, use_color=self.use_color)
         msg.body = pack('Bxxxiii', socket.AF_UNSPEC, 0, 0, 0)
         msg.flags = NLM_F_REQUEST | NLM_F_DUMP | NLM_F_ACK
-        msg.build_message(self.sequence.next(), self.pid)
+        msg.build_message(next(self.sequence), self.pid)
+        return self.tx_nlpacket_get_response(msg)
+
+    # ===
+    # MDB
+    # ===
+    def mdb_dump(self):
+        debug = RTM_GETMDB in self.debug
+        msg = MDB(RTM_GETMDB, debug, use_color=self.use_color)
+        msg.body = pack('Bxxxiii', socket.AF_BRIDGE, 0, 0, 0)
+        msg.flags = NLM_F_REQUEST | NLM_F_DUMP | NLM_F_ACK
+        msg.build_message(next(self.sequence), self.pid)
         return self.tx_nlpacket_get_response(msg)
